/*============================================================================
  File:     FilteredStatistics.sql

  Summary:  Now that we know we have skew - how can automate the
			creation of filtered statistics.
  
  SQL Server Version: SQL Server 2008+
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

---------------------------------------------
-- Setup (for filtered stats)
---------------------------------------------

-- Be sure to execute ALL of these scripts:
--  * sp_SQLskills_CreateFilteredStatsString.sql
--  * sp_SQLskills_DropAllColumnStats.sql
--	* sp_SQLskills_CreateFilteredStats.sql


---------------------------------------------
-- Quick check on stats and indexes
---------------------------------------------

USE [AdventureWorksDW2008_ModifiedSalesKey];
GO

EXEC [sp_SQLskills_SQL2012_helpindex] 'dbo.factinternetsales';
EXEC [sp_helpstats] 'dbo.FactInternetSales', 'ALL';
GO

/************************************************************
-- Procedure:[sp_SQLskills_CreateFilteredStats]

-- PARAMETER NOTES:
--
--  @filteredstats	Number of steps to create.
--					You must have at least this number of steps
--					in your histogram.

--	@everincreasing	For a column (even an FK) whose values
--					are ever-increasing in terms of inserts,
--					this should be set to 1 AND you should
--					define a value for @maxforfs

--	@maxforfs		This sets the boundary value outside
--					of the histogram. For example, if you have
--					18790 customers and you get about 100 new 
--					customers per day then you can set your
--					max based on the number of days until the
--					next redisribution (weekly/monthly).

--	@fullscan		Generate the filtered stat with a fullscan
--					or sample.
--					
--	@samplepercent	This can be null even when fullscan is 
--					SAMPLE. SQL Server will just use the internal
--					calculation for sample size.

************************************************************/

DECLARE @maxid	int
SELECT @maxid = MAX([c].[CustomerKey]) + (7*100)
FROM [dbo].[DimCustomer] AS [c]

EXEC [dbo].[sp_SQLskills_CreateFilteredStats]
		  @schemaname		= N'dbo'
		, @objectname		= N'factinternetsales'
		, @columnname		= N'customerkey'
		, @filteredstats	= 10
		, @everincreasing	= 1
		, @maxforfs			= @maxid
		, @fullscan			= 'FULLSCAN';
GO

EXEC [sp_SQLskills_SQL2012_helpindex] 'dbo.factinternetsales';
EXEC [sp_helpstats] 'dbo.FactInternetSales', 'ALL';
GO

-- check a few queries
EXEC sp_recompile 'dbo.FactInternetSales';
GO

SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s]
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = 11152
OPTION (QUERYTRACEON 3604, QUERYTRACEON 9204, RECOMPILE);
GO

SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s]
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = 11176
OPTION (QUERYTRACEON 3604, QUERYTRACEON 9204);
GO

SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s]
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = 11693
OPTION (QUERYTRACEON 3604, QUERYTRACEON 9204);
GO

SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s]
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = 11711
OPTION (QUERYTRACEON 3604, QUERYTRACEON 9204);
GO

---------------------------------------------
-- Maybe that's not enough stats?
---------------------------------------------

DECLARE @maxid	int
SELECT @maxid = MAX([c].[CustomerKey]) + (10*100)
FROM [dbo].[DimCustomer] AS [c]

EXEC [dbo].[sp_SQLskills_CreateFilteredStats]
		  @schemaname		= N'dbo'
		, @objectname		= N'factinternetsales'
		, @columnname		= N'customerkey'
		, @filteredstats	= 20
		, @everincreasing	= 1
		, @maxforfs			= @maxid
		, @fullscan			= 'FULLSCAN';
GO

EXEC [sp_SQLskills_SQL2012_helpindex] 'dbo.factinternetsales';
EXEC [sp_helpstats] 'dbo.FactInternetSales', 'ALL';
GO

-- Retry all of the queries
EXEC sp_recompile 'dbo.FactInternetSales';
GO

SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s]
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = 11152
OPTION (QUERYTRACEON 3604, QUERYTRACEON 9204);
GO

SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s]
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = 11176
OPTION (QUERYTRACEON 3604, QUERYTRACEON 9204);
GO

SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s]
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = 11693
OPTION (QUERYTRACEON 3604, QUERYTRACEON 9204);
GO

SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s]
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = 11711
OPTION (QUERYTRACEON 3604, QUERYTRACEON 9204);
GO

---------------------------------------------
-- 10-20 should be enough for most tables
-- This is giving you 10-20x the steps!
---------------------------------------------

---------------------------------------------
-- Remove the filtered stats
---------------------------------------------

-- What if you want to quickly drop all of 
-- the column-level stats that were created.

EXEC [sp_SQLskills_DropAllColumnStats]
		  @schemaname		= N'dbo'
		, @objectname		= N'factinternetsales'
		, @columnname		= N'customerkey';

-- Just to reconfirm
EXEC [sp_SQLskills_SQL2012_helpindex] 'dbo.factinternetsales';
EXEC [sp_helpstats] 'dbo.FactInternetSales', 'ALL';
GO

-- Or, if you want to remove all of the stats:

EXEC [sp_SQLskills_DropAllColumnStats]
		  @schemaname		= N'dbo'
		, @objectname		= N'factinternetsales'
		, @columnname		= N'customerkey'
		, @dropall			= N'TRUE'

-- Just to reconfirm
EXEC [sp_SQLskills_SQL2012_helpindex] 'dbo.factinternetsales';
EXEC [sp_helpstats] 'dbo.FactInternetSales', 'ALL';
GO

---------------------------------------------
-- Final notes 
---------------------------------------------

-- Play around with this IN DEVELOPMENT/TEST (er, duh) 

-- Using the create filtered index proc will DROP all other 
-- stats that are just single column-level stats on the 
-- column where you're creating filtered stats.
-- This really should not cause any issues unless you later
-- drop the filtered stats and do not have auto create
-- statistics on (and, there's no index/stats that could be
-- used). But, since we're mostly focused on creating FS
-- to help indexes then the index stats should be used 
-- and the column-level stat was just redundant anyway!

-- And, remember - these (PROBABLY) CANNOT be used without code
-- changes....
